import serial
import logging
import time

def write_and_check(socket, command, expected_reply=None, max_retry=3):
    """
    Write a command on arduino and check for the reply.

    socket -- serial socket for arduino
    command -- str
    expected_reply -- str. If None, no check performed.
    """
    command += '\n'
    logging.debug('clear buffer...')
    _ = socket.read_all()
    socket.write(command.encode())
    logging.debug(f'check for reply of command {command}')
    reply = socket.read_until()
    logging.debug(f'reply is {reply}')

    if expected_reply:
        attempt = 0
        expected_reply += '\r\n'
        while attempt < max_retry and reply != expected_reply.encode():
            logging.debug('Not the expected reply, wait...')
            time.sleep(.2)
            logging.debug('clear buffer...')
            _ = socket.read_all()
            logging.debug('retry...')
            socket.write(command.encode())
            #time.sleep(.2)
            reply = socket.read_until()
            logging.debug(f'{command} reply {reply}')
            attempt += 1
        if attempt == max_retry:
            raise RuntimeError


def write_and_read(socket, command):
    command += '\n'
    socket.write(command.encode())
    logging.debug(f'{command} sent')
    time.sleep(.2)
    reply = socket.read_until()
    reply = reply.decode()
    logging.debug(f'{command} reply {reply}')
    return reply
